function p_out = sample_pjk(r,p,gam,delta02,psi,nu,proba_b,proba_d,y);

%------------------------------------------------------------------
% This function allows to sample the AR model orders
%       according to f(p|gam,delta02,r,p,nu,xi,y)
% 
% INPUT
%       r       : the current state of the matrix R
%       p       : the current state of the model order
%       gam     : gam parameter
%       delta02 : delta02 parameter
%       psi     : psi parameter
%       nu      : nu parameter
%       proba_b : array of the probabilities of a birth move
%       proba_d : array of the probabilities of a death move
%       y       : the signal to be segmented
%
% OUTPUT
%       p_out : the new state of the AR model order
%
%------------------------------------------------------------------


global J N pmax Tepsilon p_0

% number of ruptures in each sequence
K = sum(r,2);
Kmax = max(K);
p_out = p;

T2 = zeros(J,Kmax);
det_M= zeros(J,Kmax);
InstantRupt = zeros(J,Kmax+1);
n = zeros(J,Kmax);
rtraf=[ones(J,1) r];

proba_u = 1-(proba_b+proba_d);

for j=1:J
    
    % looking for the ruptures...
    InstantRupt(j,1:K(j)+1) = [0 find(r(j,:)==1)];
    
    % the length of the segment
    n(j,1:K(j)) = diff(find(rtraf(j,:)==1));

    for k=randperm(K(j))
        % current order
        p_jk = p_out(j,k);
        
        % birth/death move ?
        move_jk = gen_discrete([-1 0 1],[proba_d(p_jk) proba_u(p_jk) proba_b(p_jk)],1,1);
        p_jk_star = p_jk + move_jk;
        
        if p_jk_star ~= p_jk

            % computation of M_jk et T_jk
            [T2_jk      det_M_jk      ] = compute_TM(InstantRupt(j,k),InstantRupt(j,k+1),p_jk,delta02,y(j,:));
            [T2_jk_star det_M_jk_star ] = compute_TM(InstantRupt(j,k),InstantRupt(j,k+1),p_jk_star,delta02,y(j,:));
            
            % acceptation probability
            log_fstar = p_jk_star*log(psi) - log(factorial(p_jk_star)) - (p_jk_star/2)*log(delta02) ...
                                           + log(det_M_jk_star)/2  - (n(j,k)/2+nu/2)*log(gam+T2_jk_star);

            log_f     = p_jk*log(psi)      - log(factorial(p_jk)) - (p_jk/2)*log(delta02) ...
                                           + log(det_M_jk)/2  - (n(j,k)/2+nu/2)*log(gam+T2_jk);
                                       
            rapport_fstar_f = exp(log_fstar-log_f);
            
            % proposal : pjk -> pjk +/- 1
            rapport_q_qstar = (proba_d(p_jk)/proba_b(p_jk))^sign(p_jk_star-p_jk);
            
            % final probability
            proba_accept_move =  min([1 rapport_fstar_f * rapport_q_qstar]);
            
            p_out(j,k) = gen_discrete([p_jk_star p_jk],[proba_accept_move 1-proba_accept_move],1,1);
            
        else
            p_out(j,k) = p_jk_star;
        end
    end
end
    