function [Tab_r, Tab_K, Tab_lambda] = segmentation(y, Nmc, Kmax, step)

%------------------------------------------------------------------
% JOINT SEGMENTATION PROCEDURE FOR POISSONIAN DISTRIBUTED DATA
% by N. DOBIGEON and J.-Y. TOURNERET 
% IRIT/ENSEEIHT/TSA - France - 2005
% 
% INPUT
%       y    : the signal to be segmented
%       Nmc   : number of MCMC iterations
%       step : Sampling step. By default equal to 1. 
%              Can be increased to accelerate the procedure
%
% OUTPUT
%       Tab_r : the samples R(t) generated by the procedure
%       Tab_K : the corresponding samples K(t)
%
%------------------------------------------------------------------

if nargin == 2
    step = 1;
end

global J N Tepsilon


% signals parameters
% J = number of sequence
% N = length of the signal
[ J N ] = size(y)

% BEGINNING OF THE SEGMENTATION
disp(' ');
disp('SEGMENTATION...');
tic;
  
% initialization of the indicator vectors
    r = [zeros(J,N-1) ones(J,1)]; 
    
% initialization of the hyperparameters
    % hyperparameters in f(r|P)
    P = ones(1,2^J); P = P/sum(P);
    
    % hyperparameters in f(lambda|nu, gam)
    nu = 2;
    gam = 2;

    % hyperparameters in f(P|alpha)
    alpha = ones(1,2^J);
     
   
% Tepsilon : all the abrupt change configurations
    eps_str=[];
    for i = 1:2^J
        eps_str = [eps_str' dec2bin(i-1,J)']';
    end
    for i = 1:2^J
        for j = 1:J
            Tepsilon(j,i) = strcmp(eps_str(i,j),'1');
        end
    end


% output : MCMC states
Tab_r = zeros(J,N,Nmc);
Tab_K = zeros(J,Nmc);
Tab_lambda = zeros(J,Kmax,Nmc);


h = waitbar(0,['Segmentation...'],'CreateCancelBtn','closereq', 'name', 'Segmentation');

for m_compt = 1:Nmc
    
     waitbar(m_compt/Nmc,h)

    % GENERATION OF R ACCORDING TO f(R|gam,y)
    
        r = sample_r(r,gam,nu,alpha,y,step);
        K = sum(r,2);
        K_max = max(K);
        Tab_r(:,:,m_compt) = r;
        Tab_K(:,m_compt) = K;
        
        
    % GENERATION OF gam ACCORDING TO  f(gam,|R,y)

        [gam lambda] = sample_gamma(r,gam,nu,y);
        for j=1:J
            Tab_lambda(j,1:K(j),m_compt) = lambda(j,1:K(j));
        end
        
end

% END OF THE SEGMENTATION
t_1 = toc;
disp('END OF THE SEGMENTATION');
disp(['Execution time : ' num2str(t_1)]);

close(h)
